@@ -1,3 +1,7 @@
+"""
+Provides defaults used throughout mod9-asr.
+"""
+
 import os
 
 MOD9_ASR_ENGINE_HOST = os.getenv('MOD9_ASR_ENGINE_HOST', 'localhost')
@@ -9,3 +13,5 @@ ENGINE_CONNECTION_RETRY_SECONDS = 10.0
 
 CHUNK_SIZE = 8 * 1024  # 8 KiB
 GS_CHUNK_SIZE = 262144  # Google requires chunks be multiples of 262144
+
+FLASK_ENV = os.getenv('FLASK_ENV', None)
