@@ -1,3 +1,8 @@
+"""
+Functions for converting Google-style and -type input and output to/from
+Mod9-style and -type.
+"""
+
 import base64
 from binascii import Error as BinasciiError
 from collections import OrderedDict
@@ -21,28 +26,31 @@ class Mod9EngineFailedStatusError(Exception):
 
 class GoogleConfigurationSettingsAndMappings:
     """
-    Hold dict's used to translate keys and values from Google to Mod9 forms.
+    Hold dicts used to translate keys and values from Google to Mod9
+    forms.
 
-    Internally use camelCase for google_allowed_keys.
-    We chose to use camelCase internally for compatibility with
-    Google's protobuf json_format funtions
-    (which are called by to_json() and from_json()).
-    We need to make sure we use the proper casing, both internally
-    and for output, or can run into non-obvious bugs.
+    Internally use camelCase for google_allowed_keys. We chose to use
+    camelCase internally for compatibility with Google's
+    ``protobuf.json_format`` funtions (which are called by ``to_json()``
+    and ``from_json()``). We need to make sure we use the proper casing,
+    both internally and for output, or can run into non-obvious bugs.
     These bugs can be non-obvious because many of the primary attributes
-    are one-word attributes (like `.transcript`) and so functionality will
-    only break if it is dependent on multi-word attributes.
+    are one-word attributes (like ``.transcript``) and so functionality
+    will only break if it is dependent on multi-word attributes.
 
-    Input from REST API come in camelCase.
-    Input from Python SDK come in snake_case, are converted to camelCase using
-    either from_json() (in the case of Google protobuf object input), or
-    utils.recursively_convert_dict_keys_case() and utils.snake_case_to_camel_case()
-    (in the case of dict input).
-    Output from REST API use camelCase.
-    Output from Python SDK converted to snake_case in to_json().
+    Input from REST API come in camelCase. Input from Python SDK come in
+    snake_case, are converted to camelCase using either ``from_json()``
+    (in the case of Google protobuf object input), or
+    ``mod9.reformat.utils.recursively_convert_dict_keys_case()`` and
+    ``reformat.utils.snake_case_to_camel_case()`` (in the case of dict
+    input). Output from REST API use camelCase. Output from Python SDK
+    converted to snake_case in ``to_json()``.
     """
     def __init__(self):
-        """Define dict's used to translate keys and values from Google to Mod9 forms."""
+        """
+        Define dicts used to translate keys and values from Google to
+        Mod9 forms.
+        """
 
         # Mod9 and Google input key names for use in following dict's.
         self.mod9_allowed_keys = (
@@ -114,7 +122,27 @@ class GoogleConfigurationSettingsAndMappings:
 
 
 def input_to_mod9(google_input_settings, module):
-    """Wrapper method to take Google inputs of various types and return Mod9-compatible inputs."""
+    """
+    Wrapper method to take Google inputs of various types and return
+    Mod9-compatible inputs.
+
+    Args:
+        google_input_settings (dict):
+            Contains dicts or Google-like-types ``.config`` and
+            ``.audio``: options for transcription and audio to be
+            transcribed, respectively.
+        module (module):
+            Module to read Google-like-types from, in case of
+            subclassing.
+
+    Returns:
+        tuple:
+            mod9_config_settings (dict):
+                Mod9-style options to pass to Mod9 ASR Engine TCP Server.
+            mod9_audio_settings (dict):
+                Mod9-style audio to pass to Mod9 ASR Engine TCP Server.
+    """
+
     # Convert keys from snake_case to camelCase (if necessary), which we use internally.
     #  See docstring for GoogleConfigurationSettingsAndMappings for more info.
     google_input_settings = utils.recursively_convert_dict_keys_case(
@@ -191,7 +219,18 @@ def input_to_mod9(google_input_settings, module):
 
 
 def google_config_settings_to_mod9(google_config_settings):
-    """Map from Google-style key:value inputs to Mod9 ASR TCP server-style key:value inputs."""
+    """
+    Map from Google-style key:value inputs to Mod9 ASR TCP server-style
+    key:value inputs.
+
+    Args:
+        google_config_settings (dict):
+            Google-style options.
+
+    Returns:
+        dict:
+            Mod9-style options to pass to Mod9 ASR Engine TCP Server.
+    """
 
     settings = GoogleConfigurationSettingsAndMappings()
     mod9_config_settings = dict()
@@ -219,7 +258,7 @@ def google_config_settings_to_mod9(google_config_settings):
             'Mod9 ASR Engine supports English at this time.'
         )
 
-    # `to_json()` populates absent attributes of the config with falsy values; remove falsy values.
+    # ``to_json()`` populates absent attributes of config with falsy values -> exceptions later.
     google_config_settings = {
         key: value for key, value in google_config_settings.items() if value
     }
@@ -251,7 +290,7 @@ def google_config_settings_to_mod9(google_config_settings):
                 settings.google_encoding_to_mod9_encoding[mod9_config_settings['encoding']]
 
     # Set N-best settings. Always get 1-best or more:
-    #  `result_from_mod9()` iterates through alternatives.
+    #  ``result_from_mod9()`` iterates through alternatives.
     mod9_config_settings['transcript-alternatives'] = True
     n_best_N = mod9_config_settings.get('transcript-alternatives-max', 1)
     # Google sets n_best_N: 0 -> 1.
@@ -266,7 +305,18 @@ def google_config_settings_to_mod9(google_config_settings):
 
 
 def google_audio_settings_to_mod9(google_audio_settings):
-    """Map from Google-style audio input to Mod9 TCP server-style audio input."""
+    """
+    Map from Google-style audio input to Mod9 TCP server-style audio
+    input.
+
+    Args:
+        google_audio_settings (dict):
+            Google-style audio.
+
+    Returns:
+        dict:
+            Mod9-style audio to pass to Mod9 ASR Engine TCP Server.
+    """
 
     if not google_audio_settings:
         return None
@@ -293,7 +343,17 @@ def google_audio_settings_to_mod9(google_audio_settings):
 
 
 def result_from_mod9(mod9_results):
-    """Map from Mod9 TCP server-style output to Google-style output."""
+    """
+    Map from Mod9 TCP server-style output to Google-style output.
+
+    Args:
+        mod9_results (Iterable[dict]):
+            Mod9-style results from the Mod9 ASR Engine TCP Server.
+
+    Yields:
+        dict:
+            Google-style result.
+    """
 
     # Longer audio comes chopped into segments.
     for mod9_result in mod9_results:
@@ -322,7 +382,7 @@ def result_from_mod9(mod9_results):
                     alternative['transcript'] += mod9_alternative['transcript']
                 alternatives.append(alternative)
         else:
-            # Partial results (`.final` == `False`) from Mod9 do not have alternatives.
+            # Partial results (``.final`` == ``False``) from Mod9 do not have alternatives.
             alternative = build_google_alternative(mod9_result['result_index'])
             alternative['transcript'] += mod9_result['transcript']
             alternatives.append(alternative)
@@ -356,7 +416,7 @@ def result_from_mod9(mod9_results):
             ]
         )
         if not mod9_result['final']:
-            google_result['stability'] = 0.5  # rather arbitrary
+            google_result['stability'] = 0.0  # Google's default.
 
         if 'phrases' in mod9_result:
             google_result['phrases'] = mod9_result['phrases']
@@ -369,7 +429,20 @@ def result_from_mod9(mod9_results):
 
 
 def build_google_alternative(transcript_number, confidence_value=1.0):
-    """Build template for Google alternative."""
+    """
+    Build template for Google alternative.
+
+    Args:
+        transcript_number (int):
+            Indicate the segment/endpoint the alternative is a part of.
+        confidence_value (float):
+            Rating in [0.0, 1.0] indicating confidence this alternative
+            is the true transcript/one-best (default is 1.0).
+
+    Returns:
+        dict:
+            Google-style transcript alternative (i.e. one of N-best list).
+    """
 
     alternative = OrderedDict([('transcript', '')])
     # Google transcripts after the first start with a space.
@@ -387,11 +460,26 @@ def google_type_result_from_dict(
         google_result_type,
         module,
 ):
-    """Convert dict-type result iterable to Google-type result generator."""
+    """
+    Convert dict-type result iterable to Google-type result generator.
+
+    Args:
+        google_result_dicts (Iterable[dict]):
+            Google-style results.
+        google_result_type (Union[module.RecognizeResponse, module.StreamingRecognizeResponse]):
+            Google-like-type to return.
+        module (module):
+            Module to read Google-like-types from, in case of
+            subclassing.
+
+    Yields:
+        Union[module.RecognizeResponse, module.StreamingRecognizeResponse]:
+            Google-like-type result.
+    """
 
     for google_result_dict in google_result_dicts:
         if google_result_type == module.SpeechRecognitionResult:
-            # `from_json()` complains if attributes that do not exist in a protobuf are passed.
+            # ``from_json()`` complains if attributes that do not exist in a protobuf are passed.
             if 'isFinal' in google_result_dict:
                 google_result_dict.pop('isFinal')
             if 'resultEndTime' in google_result_dict:
